#include "theBiasPlanet/coreUtilities/collections/NavigableLinkedMap.hpp"
#include <cstdarg>
#include <stdexcept>
#include <string>
#include "theBiasPlanet/coreUtilities/locksHandling/Locker.hpp"

using namespace ::theBiasPlanet::coreUtilities::locksHandling;

namespace theBiasPlanet {
	namespace coreUtilities {
		namespace collections {
			template <typename T, typename U, typename W> NavigableLinkedMap <T, U, W>::Links::Links () : i_previousKey (nullopt), i_nextKey (nullopt) {
			}
			
			template <typename T, typename U, typename W> NavigableLinkedMap <T, U, W>::Links::Links (optional <key_type> a_previousKey, optional <key_type> a_nextKey) : i_previousKey (a_previousKey), i_nextKey (a_nextKey) {
			}
			
			template <typename T, typename U, typename W> optional <typename NavigableLinkedMap <T, U, W>::key_type> NavigableLinkedMap <T, U, W>::Links::getPreviousKey () {
				return i_previousKey;
			}
			
			template <typename T, typename U, typename W> void NavigableLinkedMap <T, U, W>::Links::setPreviousKey (optional <key_type> a_previousKey) {
				i_previousKey = a_previousKey;
			}
			
			template <typename T, typename U, typename W> optional <typename NavigableLinkedMap <T, U, W>::key_type> NavigableLinkedMap <T, U, W>::Links::getNextKey () {
				return i_nextKey;
			}
			
			template <typename T, typename U, typename W> void NavigableLinkedMap <T, U, W>::Links::setNextKey (optional <key_type> a_nextKey) {
				i_nextKey = a_nextKey;
			}
			
			template <typename T, typename U, typename W> pair <typename NavigableLinkedMap <T, U, W>::key_type &, typename NavigableLinkedMap <T, U, W>::mapped_type &> NavigableLinkedMap <T, U, W>::getKeyValuePair (key_type & a_key, mapped_type & a_value) {
				return pair <key_type &, mapped_type &> (a_key, a_value);
			}
			
			template <typename T, typename U, typename W> NavigableLinkedMap <T, U, W>::ValueComparer::ValueComparer (key_compare a_keyComparer) : i_keyComparer (a_keyComparer) {
			}
			
			template <typename T, typename U, typename W> bool NavigableLinkedMap <T, U, W>::ValueComparer::operator () (value_type const & a_keyValuePair1, value_type const & a_keyValuePair2) const {
				return i_keyComparer (a_keyValuePair1.first, a_keyValuePair2.first);
			}
			
			template <typename T, typename U, typename W> void NavigableLinkedMap <T, U, W>::insertNewElementAtEnd (key_type const a_key, optional <mapped_type> a_value) {
				mapped_type & l_insertedElementValue = (*i_keyToValueMap) [a_key];
				Links & l_insertedElementLinks = (*i_keyToLinksMap) [a_key];
				if (i_lastKey->has_value ()) {
					(*i_keyToLinksMap) [**i_lastKey].setNextKey (a_key);
				}
				l_insertedElementLinks.setPreviousKey (*i_lastKey); 
				*i_lastKey = a_key;
				if (!i_firstKey->has_value ()) {
					*i_firstKey = a_key;
				}
				if (a_value.has_value ()) {
					l_insertedElementValue = *a_value;	
				}
			}
			
			template <typename T, typename U, typename W> void NavigableLinkedMap <T, U, W>::insertNewElementAtMiddle (key_type const a_positionKey, key_type const a_key, optional <mapped_type> a_value) {
				Links & l_foundElementLinks = (*i_keyToLinksMap) [a_positionKey];
				mapped_type & l_insertedElementValue = (*i_keyToValueMap) [a_key];
				Links & l_insertedElementLinks = (*i_keyToLinksMap) [a_key];
				optional <key_type> l_previousKey = l_foundElementLinks.getPreviousKey ();
				if (l_previousKey == nullopt) {
					*i_firstKey = a_key;
				}
				else {
					Links & l_previousElementLinks = (*i_keyToLinksMap) [*l_previousKey];
					l_previousElementLinks.setNextKey (a_key);
				}
				l_insertedElementLinks.setPreviousKey (l_foundElementLinks.getPreviousKey ()); 
				l_insertedElementLinks.setNextKey (a_positionKey); 
				l_insertedElementValue = *a_value; 
				l_foundElementLinks.setPreviousKey (a_key); 
			}
			
			template <typename T, typename U, typename W> void NavigableLinkedMap <T, U, W>::updateElement (key_type const a_key, mapped_type a_value) {
				mapped_type & l_updatedElementValue = i_keyToValueMap->at (a_key);
				l_updatedElementValue = a_value; 
			}
			
			template <typename T, typename U, typename W> NavigableLinkedMap <T, U, W>::NavigableLinkedMap (key_compare const & a_keysComparer) : i_keyToValueMap (new map <key_type, mapped_type, key_compare> (a_keysComparer)), i_keyToLinksMap (new map <key_type, Links, key_compare> ( (key_compare) a_keysComparer)), i_firstKey (new optional <key_type> ()), i_lastKey (new optional <key_type> ()) {
			}
			
			template <typename T, typename U, typename W> template <typename V> NavigableLinkedMap <T, U, W>::NavigableLinkedMap (V a_iteratorPointedAtFirstElement, V a_iteratorPointedAtLastElementNotIncluded, key_compare const & a_keysComparer) : i_keyToValueMap (new map <key_type, mapped_type, key_compare> (a_keysComparer)), i_keyToLinksMap (new map <key_type, Links, key_compare> ( (key_compare) a_keysComparer)), i_firstKey (new optional <key_type> ()), i_lastKey (new optional <key_type> ()) {
				this->insert (a_iteratorPointedAtFirstElement, a_iteratorPointedAtLastElementNotIncluded);
			}
			
			template <typename T, typename U, typename W> NavigableLinkedMap <T, U, W>::NavigableLinkedMap (initializer_list <value_type> a_initializer, key_compare const & a_keysComparer) : i_keyToValueMap (new map <key_type, mapped_type, key_compare> (a_keysComparer)), i_keyToLinksMap (new map <key_type, Links, key_compare> ( (key_compare) a_keysComparer)), i_firstKey (new optional <key_type> ()), i_lastKey (new optional <key_type> ()) {
				for (value_type const & l_keyValuePair: a_initializer) {
					insert (l_keyValuePair);
				}
			}
			
			template <typename T, typename U, typename W> NavigableLinkedMap <T, U, W>::NavigableLinkedMap (NavigableLinkedMap <key_type, mapped_type, key_compare> const & a_navigableLinkedMap) : i_keyToValueMap (new map <key_type, mapped_type, key_compare> (*(a_navigableLinkedMap.i_keyToValueMap))), i_keyToLinksMap (new map <key_type, Links, key_compare> (*(a_navigableLinkedMap.i_keyToLinksMap))), i_firstKey (new optional <key_type> (*(a_navigableLinkedMap.i_firstKey))), i_lastKey (new optional <key_type> (*(a_navigableLinkedMap.i_lastKey))) {
			}
			
			template <typename T, typename U, typename W> NavigableLinkedMap <T, U, W>::~NavigableLinkedMap () {
				delete i_keyToValueMap;
				i_keyToValueMap = nullptr;
				delete i_keyToLinksMap;
				i_keyToLinksMap = nullptr;
				delete i_firstKey;
				i_firstKey = nullptr;
				delete i_lastKey;
				i_lastKey = nullptr;
			}
			
			template <typename T, typename U, typename W> NavigableLinkedMap <T, U, W> & NavigableLinkedMap <T, U, W>::operator = (NavigableLinkedMap <key_type, mapped_type, key_compare> const & a_navigableLinkedMap) {
				*i_keyToValueMap = *(a_navigableLinkedMap.i_keyToValueMap);
				*i_keyToLinksMap = *(a_navigableLinkedMap.i_keyToLinksMap);
				*i_firstKey = *(a_navigableLinkedMap.i_firstKey);
				*i_lastKey = *(a_navigableLinkedMap.i_lastKey);
				return *this;
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::iterator NavigableLinkedMap <T, U, W>::begin () {
				return iterator (*this, *i_firstKey);
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::const_iterator NavigableLinkedMap <T, U, W>::begin () const {
				return const_iterator (*this, *i_firstKey);
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::iterator NavigableLinkedMap <T, U, W>::end () {
				return iterator (*this, nullopt);
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::const_iterator NavigableLinkedMap <T, U, W>::end () const {
				return const_iterator (*this, nullopt);
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::reverse_iterator NavigableLinkedMap <T, U, W>::rbegin () {
				return reverse_iterator (end ());
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::const_reverse_iterator NavigableLinkedMap <T, U, W>::rbegin () const {
				return const_reverse_iterator (end ());
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::reverse_iterator NavigableLinkedMap <T, U, W>::rend () {
				return reverse_iterator (begin ());
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::const_reverse_iterator NavigableLinkedMap <T, U, W>::rend () const {
				return const_reverse_iterator (begin ());
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::const_iterator NavigableLinkedMap <T, U, W>::cbegin () const noexcept {
				return const_iterator (*this, *i_firstKey);
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::const_iterator NavigableLinkedMap <T, U, W>::cend () const noexcept {
				return const_iterator (*this, nullopt);
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::const_reverse_iterator NavigableLinkedMap <T, U, W>::crbegin () const noexcept {
				return const_reverse_iterator (cend ());
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::const_reverse_iterator NavigableLinkedMap <T, U, W>::crend () const noexcept {
				return const_reverse_iterator (cbegin ());
			}
			
			template <typename T, typename U, typename W> bool NavigableLinkedMap <T, U, W>::empty () const {
				i_keyToValueMap->empty ();
				return i_keyToLinksMap->empty ();
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::size_type NavigableLinkedMap <T, U, W>::size () const {
				return i_keyToValueMap->size ();
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::size_type NavigableLinkedMap <T, U, W>::max_size () const {
				return i_keyToValueMap->max_size ();
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::mapped_type & NavigableLinkedMap <T, U, W>::operator []  (key_type const & a_key) {
				Locker l_locker (&i_mutex) ;
				if (i_keyToValueMap->count (a_key) == 0) {
					insertNewElementAtEnd (a_key, nullopt);
				}
				else {
				}
				return (*i_keyToValueMap) [a_key];
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::mapped_type & NavigableLinkedMap <T, U, W>::at (key_type const & a_key) {
				return i_keyToValueMap->at (a_key);
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::mapped_type const & NavigableLinkedMap <T, U, W>::at (key_type const & a_key) const {
				return i_keyToValueMap->at (a_key);
			}
			
			template <typename T, typename U, typename W> pair <typename NavigableLinkedMap <T, U, W>::iterator, bool> NavigableLinkedMap <T, U, W>::insert (value_type const & a_keyValuePair) {
				Locker l_locker (&i_mutex);
				bool l_isNewlyInserted = false;
				if (i_keyToValueMap->count (a_keyValuePair.first) == 0) {
					l_isNewlyInserted = true;
					insertNewElementAtEnd (a_keyValuePair.first, a_keyValuePair.second);
				}
				else {
					updateElement (a_keyValuePair.first, a_keyValuePair.second);
				}
				return pair (iterator (*this, a_keyValuePair.first), l_isNewlyInserted);
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::iterator NavigableLinkedMap <T, U, W>::insert (iterator a_position, value_type const & a_keyValuePair) {
				Locker l_locker (&i_mutex);
				if (i_keyToValueMap->count (a_keyValuePair.first) == 0) {
					if (i_keyToValueMap->count ((*a_position).first) == 0) {
						insertNewElementAtEnd (a_keyValuePair.first, a_keyValuePair.second);
					}
					else {
						insertNewElementAtMiddle ((*a_position).first, a_keyValuePair.first, a_keyValuePair.second);
					}
				}
				else {
					updateElement (a_keyValuePair.first, a_keyValuePair.second);
				}
				return iterator (*this, a_keyValuePair.first);
			}
			
			template <typename T, typename U, typename W> template <typename V> void NavigableLinkedMap <T, U, W>::insert (V a_iteratorPointedAtFirstElement, V a_iteratorPointedAtLastElementNotIncluded) {
				for (V l_iterator = a_iteratorPointedAtFirstElement; l_iterator != a_iteratorPointedAtLastElementNotIncluded; l_iterator++) {
					insert (*l_iterator);
				}
			}
			
			template <typename T, typename U, typename W> void NavigableLinkedMap <T, U, W>::erase (iterator a_position) {
				erase ((*a_position).first);
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::size_type NavigableLinkedMap <T, U, W>::erase (key_type const & a_key) {
				Locker l_locker (&i_mutex);
				if (i_keyToValueMap->count (a_key) == 0) {
					return 0;
				}
				else {
					Links & l_foundElementLinks = (*i_keyToLinksMap) [a_key];
					optional <key_type> l_previousKey = l_foundElementLinks.getPreviousKey (); 
					optional <key_type> l_nextKey = l_foundElementLinks.getNextKey (); 
					if (l_previousKey.has_value ()) {
						Links & l_previousElementLinks = (*i_keyToLinksMap) [*l_previousKey];
						if (l_nextKey.has_value ()) {
							Links & l_nextElementLinks = (*i_keyToLinksMap) [*l_nextKey];
							l_previousElementLinks.setNextKey (l_nextKey);
							l_nextElementLinks.setPreviousKey (l_previousKey);
						}
						else {
							l_previousElementLinks.setNextKey (nullopt);
							*i_lastKey = l_previousKey;
						}
					}
					else {
						if (l_nextKey.has_value ()) {
							Links & l_nextElementLinks = (*i_keyToLinksMap) [*l_nextKey];
							*i_firstKey = l_nextKey;
							l_nextElementLinks.setPreviousKey (nullopt);
						}
						else {
							*i_firstKey = nullopt;
							*i_lastKey = nullopt;
						}
					}
					i_keyToValueMap->erase (a_key);
					i_keyToLinksMap->erase (a_key);
					return 1;
				}
			}
			
     		template <typename T, typename U, typename W> void NavigableLinkedMap <T, U, W>::erase (iterator a_iteratorPointedAtFirstElement, iterator a_iteratorPointedAtLastElementNotIncluded) {
				for (NavigableLinkedMap <key_type, mapped_type, key_compare>::iterator l_iterator = a_iteratorPointedAtFirstElement; l_iterator != a_iteratorPointedAtLastElementNotIncluded; l_iterator++) {
					erase (l_iterator);
				}
			}
			
			template <typename T, typename U, typename W> void NavigableLinkedMap <T, U, W>::swap (NavigableLinkedMap <key_type, mapped_type, key_compare> & a_navigableLinkedMap) {
				map <key_type, mapped_type, key_compare> * l_keyToValueMapSaved = i_keyToValueMap;
				map <key_type, Links, key_compare> * l_keyToLinksMapSaved = i_keyToLinksMap;
				i_keyToValueMap = a_navigableLinkedMap.i_keyToValueMap;
				i_keyToLinksMap = a_navigableLinkedMap.i_keyToLinksMap;
				a_navigableLinkedMap.i_keyToValueMap = l_keyToValueMapSaved;
				a_navigableLinkedMap.i_keyToLinksMap = l_keyToLinksMapSaved;
				optional <key_type> * l_firstKeySaved = i_firstKey;
				i_firstKey = a_navigableLinkedMap.i_firstKey;
				a_navigableLinkedMap.i_firstKey = l_firstKeySaved;
				optional <key_type> * l_lastKeySaved = i_lastKey;
				i_lastKey = a_navigableLinkedMap.i_lastKey;
				a_navigableLinkedMap.i_lastKey = l_lastKeySaved;
			}
			
			template <typename T, typename U, typename W> void NavigableLinkedMap <T, U, W>::clear () {
				i_keyToValueMap->clear ();
				i_keyToLinksMap->clear ();
				*i_firstKey = nullopt;
				*i_lastKey = nullopt;
			}
			
			template <typename T, typename U, typename W> template <typename ... V> pair <typename NavigableLinkedMap <T, U, W>::iterator, bool> NavigableLinkedMap <T, U, W>::emplace (V && ... a_arguments) {
				Locker l_locker (&i_mutex);
				pair <key_type &, mapped_type &> l_keyValuePair = getKeyValuePair (a_arguments...);
				T & l_key = l_keyValuePair.first;
				U & l_value = l_keyValuePair.second;
				bool l_isNewlyInserted = false;
				if (i_keyToValueMap->count (l_key) == 0) {
					l_isNewlyInserted = true;
					if (i_lastKey->has_value ()) {
						(*i_keyToLinksMap) [**i_lastKey].setNextKey (l_key);
					}
					i_keyToValueMap->emplace (l_key, l_value);
					i_keyToLinksMap->emplace (l_key, Links (*i_lastKey, nullopt));
					*i_lastKey = l_key;
					if (!i_firstKey->has_value ()) {
						*i_firstKey = l_key;
					}
				}
				else {
				}
				return pair (iterator (*this, l_key), l_isNewlyInserted);
			}
			
			template <typename T, typename U, typename W> template <typename ... V> typename NavigableLinkedMap <T, U, W>::iterator NavigableLinkedMap <T, U, W>::emplace_hint (const_iterator a_position, V && ... a_arguments) {
				Locker l_locker (&i_mutex);
				pair <key_type &, mapped_type &> l_keyValuePair = getKeyValuePair (a_arguments...);
				key_type & l_key = l_keyValuePair.first;
				mapped_type & l_value = l_keyValuePair.second;
				bool l_isNewlyInserted = false;
				if (i_keyToValueMap->count (l_key) == 0) {
					l_isNewlyInserted = true;
					if (i_keyToValueMap->count (a_position->first) == 0) {
						return emplace (a_arguments...).first;
					}
					else {
						Links & l_foundElementLinks = (*i_keyToLinksMap) [a_position->first];
						optional <key_type> l_previousKey = l_foundElementLinks.getPreviousKey ();
						i_keyToValueMap->emplace (l_key, l_value);
						i_keyToLinksMap->emplace (l_key, Links (l_previousKey, a_position->first));
						if (!l_previousKey.has_value ()) {
							*i_firstKey = l_key;
						}
						else {
							Links & l_previousElementLinks = (*i_keyToLinksMap) [*l_previousKey];
							l_previousElementLinks.setNextKey (l_key); 
						}
						l_foundElementLinks.setPreviousKey (l_key); 
					}
				}
				else {
				}
				return iterator (*this, l_key);
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::key_compare NavigableLinkedMap <T, U, W>::key_comp () const {
				return i_keyToValueMap->key_comp ();
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::value_compare NavigableLinkedMap <T, U, W>::value_comp () const {
				return value_compare (key_comp ());
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::iterator NavigableLinkedMap <T, U, W>::find (key_type const & a_key) {
				if (i_keyToValueMap->count (a_key) == 0) {
					return iterator (*this, nullopt);
				}
				else {
					return iterator (*this, a_key);
				}
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::const_iterator NavigableLinkedMap <T, U, W>::find (key_type const & a_key) const {
				if (i_keyToValueMap->count (a_key) == 0) {
					return const_iterator (*this, nullopt);
				}
				else {
					return const_iterator (*this, a_key);
				}
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::size_type NavigableLinkedMap <T, U, W>::count (key_type const & a_key) const {
				return i_keyToValueMap->count (a_key);
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::iterator NavigableLinkedMap <T, U, W>::lower_bound (key_type const & a_key) {
				key_compare l_keyComparer = this->key_comp ();
				for (value_type const & l_keyValuePair: *this) {
					if (!l_keyComparer (l_keyValuePair.first, a_key)) {
						return iterator (*this, l_keyValuePair.first); 
					}
				}
				return end ();
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::const_iterator NavigableLinkedMap <T, U, W>::lower_bound (key_type const & a_key) const {
				key_compare l_keyComparer = this->key_comp ();
				for (value_type const & l_keyValuePair: *this) {
					if (!l_keyComparer (l_keyValuePair.first, a_key)) {
						return const_iterator (*this, l_keyValuePair.first); 
					}
				}
				return cend ();
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::iterator NavigableLinkedMap <T, U, W>::upper_bound (key_type const & a_key) {
				key_compare l_keyComparer = this->key_comp ();
				for (value_type const & l_keyValuePair: *this) {
					if (l_keyComparer (a_key, l_keyValuePair.first)) {
						return iterator (*this, l_keyValuePair.first); 
					}
				}
				return end ();
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::const_iterator NavigableLinkedMap <T, U, W>::upper_bound (key_type const & a_key) const {
				key_compare l_keyComparer = this->key_comp ();
				for (value_type const & l_keyValuePair: *this) {
					if (l_keyComparer (a_key, l_keyValuePair.first)) {
						return const_iterator (*this, l_keyValuePair.first); 
					}
				}
				return cend ();
			}
			
			template <typename T, typename U, typename W> pair <typename NavigableLinkedMap <T, U, W>::const_iterator, typename NavigableLinkedMap <T, U, W>::const_iterator> NavigableLinkedMap <T, U, W>::equal_range (key_type const & a_key) const {
				const_iterator l_lowerBoundIterator = lower_bound (a_key);
				if (l_lowerBoundIterator != cend () && l_lowerBoundIterator->first == a_key) {
					iterator l_nextToLowerBoundIterator = l_lowerBoundIterator;
					l_nextToLowerBoundIterator ++;
					return pair <iterator, iterator> (l_lowerBoundIterator, l_nextToLowerBoundIterator);
				}
				else {
					return pair <const_iterator, const_iterator> (l_lowerBoundIterator, l_lowerBoundIterator);
				}
			}
			
			template <typename T, typename U, typename W> pair <typename NavigableLinkedMap <T, U, W>::iterator, typename NavigableLinkedMap <T, U, W>::iterator> NavigableLinkedMap <T, U, W>::equal_range (key_type const & a_key) {
				iterator l_lowerBoundIterator = lower_bound (a_key);
				if (l_lowerBoundIterator != end () && l_lowerBoundIterator->first == a_key) {
					iterator l_nextToLowerBoundIterator = l_lowerBoundIterator;
					l_nextToLowerBoundIterator ++;
					return pair <iterator, iterator> (l_lowerBoundIterator, l_nextToLowerBoundIterator);
				}
				else {
					return pair <iterator, iterator> (l_lowerBoundIterator, l_lowerBoundIterator);
				}
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::allocator_type NavigableLinkedMap <T, U, W>::get_allocator () const {
				return i_keyToValueMap->get_allocator ();
			}
			
			template <typename T, typename U, typename W> void NavigableLinkedMap <T, U, W>::BaseIterator::goToNextElement () {
				if (i_currentKey.has_value ()) {
					try {
						Links & l_currentElementLinks = i_keyToLinksMap->at (*i_currentKey);
						i_currentKey = l_currentElementLinks.getNextKey ();
					}
					catch (out_of_range l_exception) {
						i_currentKey  = nullopt;
					}
				}
				else {
				}
			}
			
			template <typename T, typename U, typename W> void NavigableLinkedMap <T, U, W>::BaseIterator::goToPreviousElement () {
				if (i_currentKey.has_value ()) {
					try {
						Links & l_currentElementLinks = i_keyToLinksMap->at (*i_currentKey);
						// If the current element is the first element, the past-the-end-element becomes the current element.
						i_currentKey = l_currentElementLinks.getPreviousKey ();
					}
					catch (out_of_range l_exception) {
						i_currentKey  = nullopt;
					}
				}
				else {
					i_currentKey  = *i_lastKey;
				}
			}
			
			template <typename T, typename U, typename W> NavigableLinkedMap <T, U, W>::BaseIterator::BaseIterator (NavigableLinkedMap <key_type, mapped_type, key_compare> const & a_navigableLinkedMap, optional <key_type> a_currentKey) : i_keyToValueMap (a_navigableLinkedMap.i_keyToValueMap), i_keyToLinksMap (a_navigableLinkedMap.i_keyToLinksMap), i_firstKey (a_navigableLinkedMap.i_firstKey), i_lastKey (a_navigableLinkedMap.i_lastKey), i_currentKey (a_currentKey) {
			}
			
			template <typename T, typename U, typename W> NavigableLinkedMap <T, U, W>::BaseIterator::BaseIterator (BaseIterator const & a_originalIterator) : i_keyToValueMap (a_originalIterator.i_keyToValueMap), i_keyToLinksMap (a_originalIterator.i_keyToLinksMap), i_firstKey (a_originalIterator.i_firstKey), i_lastKey (a_originalIterator.i_lastKey), i_currentKey (a_originalIterator.i_currentKey) {
			}
			
			template <typename T, typename U, typename W> bool NavigableLinkedMap <T, U, W>::BaseIterator::operator == (BaseIterator const & a_iteratorToBeCompared) const {
				return (i_currentKey == a_iteratorToBeCompared.i_currentKey);
			}
			
			template <typename T, typename U, typename W> bool NavigableLinkedMap <T, U, W>::BaseIterator::operator != (BaseIterator const & a_iteratorToBeCompared) const {
				return (i_currentKey != a_iteratorToBeCompared.i_currentKey);
			}
			
			template <typename T, typename U, typename W> NavigableLinkedMap <T, U, W>::NonConstantIterator::NonConstantIterator (NavigableLinkedMap <key_type, mapped_type, key_compare> const & a_navigableLinkedMap, optional <key_type> a_currentKey) : BaseIterator (a_navigableLinkedMap, a_currentKey) {
			}
			
			template <typename T, typename U, typename W> NavigableLinkedMap <T, U, W>::NonConstantIterator::NonConstantIterator (BaseIterator const & a_originalIterator) : BaseIterator (a_originalIterator) {
			}
							
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::NonConstantIterator & NavigableLinkedMap <T, U, W>::NonConstantIterator::operator ++ () {
				this->goToNextElement ();
				return *this;
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::NonConstantIterator NavigableLinkedMap <T, U, W>::NonConstantIterator::operator ++ (int) {
				NonConstantIterator l_copiedIterator (*this);
				this->goToNextElement ();
				return l_copiedIterator;
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::NonConstantIterator & NavigableLinkedMap <T, U, W>::NonConstantIterator::operator -- () {
				this->goToPreviousElement ();
				return *this;
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::NonConstantIterator NavigableLinkedMap <T, U, W>::NonConstantIterator::operator -- (int) {
				NonConstantIterator l_copiedIterator (*this);
				this->goToPreviousElement ();
				return l_copiedIterator;
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::NonConstantIterator::value_type & NavigableLinkedMap <T, U, W>::NonConstantIterator::operator * () {
				if (this->i_currentKey.has_value ()) {
					try {
						return this->i_keyToValueMap->find (*(this->i_currentKey)).operator * ();
					}
					catch (out_of_range l_exception) {
						throw l_exception;
					}
				}
				else {
					throw out_of_range ("The key does not exist.");
				}
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::NonConstantIterator::value_type * NavigableLinkedMap <T, U, W>::NonConstantIterator::operator -> () {
				if (this->i_currentKey.has_value ()) {
					try {
						return this->i_keyToValueMap->find (*(this->i_currentKey)).operator -> ();
						
					}
					catch (out_of_range l_exception) {
						throw l_exception;
					}
				}
				else {
					throw out_of_range ("The key does not exist.");
				}
			}
			
			template <typename T, typename U, typename W> NavigableLinkedMap <T, U, W>::ConstantIterator::ConstantIterator (NavigableLinkedMap <key_type, mapped_type, key_compare> const & a_navigableLinkedMap, optional <key_type> a_currentKey) : BaseIterator (a_navigableLinkedMap, a_currentKey) {
			}
			
			template <typename T, typename U, typename W> NavigableLinkedMap <T, U, W>::ConstantIterator::ConstantIterator (BaseIterator const & a_originalIterator) : BaseIterator (a_originalIterator) {
			}
							
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::ConstantIterator & NavigableLinkedMap <T, U, W>::ConstantIterator::operator ++ () {
				this->goToNextElement ();
				return *this;
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::ConstantIterator NavigableLinkedMap <T, U, W>::ConstantIterator::operator ++ (int) {
				ConstantIterator l_copiedIterator (*this);
				this->goToNextElement ();
				return l_copiedIterator;
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::ConstantIterator & NavigableLinkedMap <T, U, W>::ConstantIterator::operator -- () {
				this->goToPreviousElement ();
				return *this;
			}
			
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::ConstantIterator NavigableLinkedMap <T, U, W>::ConstantIterator::operator -- (int) {
				ConstantIterator l_copiedIterator (*this);
				this->goToPreviousElement ();
				return l_copiedIterator;
			}
			
#ifdef GCC
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::ConstantIterator::value_type & NavigableLinkedMap <T, U, W>::ConstantIterator::operator * () {
#else
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::ConstantIterator::NonConstantvalue_type const & NavigableLinkedMap <T, U, W>::ConstantIterator::operator * () {
#endif
				if (this->i_currentKey.has_value ()) {
					try {
						return (const_cast <map <key_type, mapped_type, key_compare> const *> (this->i_keyToValueMap))->find (*(this->i_currentKey)).operator * ();
					}
					catch (out_of_range l_exception) {
						throw l_exception;
					}
				}
				else {
					throw out_of_range ("The key does not exist.");
				}
			}
			
#ifdef GCC
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::ConstantIterator::value_type * NavigableLinkedMap <T, U, W>::ConstantIterator::operator -> () {
#else
			template <typename T, typename U, typename W> typename NavigableLinkedMap <T, U, W>::ConstantIterator::NonConstantvalue_type const * NavigableLinkedMap <T, U, W>::ConstantIterator::operator -> () {
#endif
				if (this->i_currentKey.has_value ()) {
					try {
						return (const_cast <map <key_type, mapped_type, key_compare> const *> (this->i_keyToValueMap))->find (*(this->i_currentKey)).operator -> ();
						
					}
					catch (out_of_range l_exception) {
						throw l_exception;
					}
				}
				else {
					throw out_of_range ("The key does not exist.");
				}
			}
		}
	}
}

